/**------------------------------------*------------------------------------**
    RELEASED TO THE PUBLIC DOMAIN BY CODEWORKS DEVELOPMENT, DECEMBER 1993.
    You may freely copy, distribute and reuse the code in this example. Codeworks disclaims any warranty of any kind, expressed or implied, as to its fitness for any particular use.
    Author:  Andrew Vyrros, av@codeworks.com
 **------------------------------------*------------------------------------**/
/**------------------------------------*------------------------------------**
    File:    Checkerboard.m
    Project: DynaDoodle/Checkerboard.bproj
    Info:    Implementation of the Checkerboard class.
 **------------------------------------*------------------------------------**/

#import "Checkerboard.h"
#import <appkit/appkit.h>


#define NAME NXLocalizedStringFromTableInBundle("Doodle.strings", [NXBundle bundleForClass:[Checkerboard class]], "Name", "Checkerboard", "Name of Checkerboard Doodle")
#define DESCRIPTION NXLocalizedStringFromTableInBundle("Doodle.strings", [NXBundle bundleForClass:[Checkerboard class]], "Description", "An alternating checkerboard pattern. You can adjust the number of tiles in each direction.", "Description of Checkerboard Doodle")


@implementation Checkerboard


- initFrame:(const NXRect *)f
{
  [super initFrame:f];
  numRows = 2;
  numCols = 2;
  return self;
}


- didLoadNib
{
  numRows = [numRowsSlider intValue];
  numCols = [numColsSlider intValue];
  return self;
}


- drawSelf:(const NXRect *)rects :(int)numRects
{
  int                 row, col;
  NXCoord             xStart;
  NXRect              tile;
  NXPoint             step;
  
  NXSetColor(backgroundColor);
  NXRectFill(&bounds);
  NXSetColor(foregroundColor);
  
  xStart = bounds.origin.x;
  tile.origin.y = bounds.origin.y;
  tile.size.width = bounds.size.width / numCols;
  tile.size.height = bounds.size.height / numRows;
  step.x = tile.size.width;
  step.y = tile.size.height;

  for (row = 0; row < numRows; row++)
  {
    if (!(row % 2))
    {
      tile.origin.x = xStart;
      col = 0;
    }
    else
   {
      tile.origin.x = xStart + step.x;
      col = 1;
    }
 
    for (; col < numCols; col += 2)
    {
      NXRectFill(&tile);
      tile.origin.x += step.x * 2.0;
    }
    tile.origin.y += step.y;
  }
  
  return self;
}


- takeNumRowsFrom:sender
{
  numRows = [[sender selectedCell] intValue];
  [self update];
  return self;
}


- takeNumColsFrom:sender
{
  numCols = [[sender selectedCell] intValue];
  [self update];
  return self;
}


@end
