/**------------------------------------*------------------------------------**
    RELEASED TO THE PUBLIC DOMAIN BY CODEWORKS DEVELOPMENT, DECEMBER 1993.
    You may freely copy, distribute and reuse the code in this example. Codeworks disclaims any warranty of any kind, expressed or implied, as to its fitness for any particular use.
    Author:  Andrew Vyrros, av@codeworks.com
 **------------------------------------*------------------------------------**/
/**------------------------------------*------------------------------------**
    File:    EyePair.m
    Project: DynaDoodle/Face.bproj
    Info:    Implementation of the EyePair class. EyePair is a helper class to Face. All that was required to add it to this module was to drag it to the classes Browser in Project Builder. Face is free to use this class as it needs. The only caveat is that Face.m must be the first class listed in PB's Classes browser. This makes Face the principal class of the module, so that DoodleBundle can find Face when it loads the code file.
 **------------------------------------*------------------------------------**/

#import "EyePair.h"
#import <appkit/appkit.h>


#define CENTERRATIO   0.333333
#define RADIUSRATIO   0.125
#define ELLIPSEFACTOR 0.552277


@implementation EyePair


- init
{
  [super init];
  pupilSize = 0.5;
  return self;
}


- setPupilSize:(float)ps
{
  pupilSize = ps;
  return self;
}


- drawSelf:(const NXRect *)rect;
{
  NXPoint center;
  NXCoord radius;
  NXSize  edgeSize, controlSize;
  
  center.x = rect->origin.x + rect->size.width * CENTERRATIO;
  center.y = rect->origin.y + rect->size.height * CENTERRATIO;
  radius = rect->size.height * RADIUSRATIO;
  edgeSize.width = radius;
  edgeSize.height = radius * pupilSize;
  controlSize.width = edgeSize.width * ELLIPSEFACTOR;
  controlSize.height = edgeSize.height * ELLIPSEFACTOR;
  
  PSmoveto(center.x - edgeSize.width, center.y);
  PScurveto(center.x - edgeSize.width, center.y + controlSize.height,
            center.x - controlSize.width, center.y + edgeSize.height,
            center.x, center.y + edgeSize.height);
  PScurveto(center.x + controlSize.width, center.y + edgeSize.height,
            center.x + edgeSize.width, center.y + controlSize.height,
            center.x + edgeSize.width, center.y);
  PScurveto(center.x + edgeSize.width, center.y - controlSize.height,
            center.x + controlSize.width, center.y - edgeSize.height,
            center.x, center.y - edgeSize.height);
  PScurveto(center.x - controlSize.width, center.y - edgeSize.height,
            center.x - edgeSize.width, center.y - controlSize.height,
            center.x - edgeSize.width, center.y);
  PSclosepath();
  PSfill();
  
  center.x = rect->origin.x + rect->size.width * 0.666667;
  PSmoveto(center.x - edgeSize.width, center.y);
  PScurveto(center.x - edgeSize.width, center.y + controlSize.height,
            center.x - controlSize.width, center.y + edgeSize.height,
            center.x, center.y + edgeSize.height);
  PScurveto(center.x + controlSize.width, center.y + edgeSize.height,
            center.x + edgeSize.width, center.y + controlSize.height,
            center.x + edgeSize.width, center.y);
  PScurveto(center.x + edgeSize.width, center.y - controlSize.height,
            center.x + controlSize.width, center.y - edgeSize.height,
            center.x, center.y - edgeSize.height);
  PScurveto(center.x - controlSize.width, center.y - edgeSize.height,
            center.x - edgeSize.width, center.y - controlSize.height,
            center.x - edgeSize.width, center.y);
  PSclosepath();
  PSfill();
    
  return self;
}


@end
