/**------------------------------------*------------------------------------**
    RELEASED TO THE PUBLIC DOMAIN BY CODEWORKS DEVELOPMENT, DECEMBER 1993.
    You may freely copy, distribute and reuse the code in this example. Codeworks disclaims any warranty of any kind, expressed or implied, as to its fitness for any particular use.
    Author:  Andrew Vyrros, av@codeworks.com
 **------------------------------------*------------------------------------**/
/**------------------------------------*------------------------------------**
    File:    Face.m
    Project: DynaDoodle/Face.bproj
    Info:    Implementation of the Face class. Face is the principal class in the module. It uses EyePair and Mouth as helper classes. All that was required to add it to the other classes to this module was to drag them to the classes Browser in Project Builder. Face is free to use these classes as it needs. The only caveat is that Face.m must be the first class listed in PB's Classes browser. This makes Face the principal class of the module, so that DoodleBundle can find Face when it loads the code file.
 **------------------------------------*------------------------------------**/

#import "Face.h"
#import "EyePair.h"
#import "Mouth.h"
#import <appkit/appkit.h>


#define NAME NXLocalizedStringFromTableInBundle("Doodle.strings", [NXBundle bundleForClass:[Face class]], "Name", "Smiley Face", "Name of Face Doodle")
#define DESCRIPTION NXLocalizedStringFromTableInBundle("Doodle.strings", [NXBundle bundleForClass:[Face class]], "Description", "A classic 1970's smiley-face. You can control the mood via the eye pupil size and the smile factor.", "Description of Face Doodle")
#define LINEWIDTHFACTOR 15.0


@implementation Face


- initFrame:(const NXRect *)f
{
  [super initFrame:f];
  eyePair = [[EyePair allocFromZone:[self zone]] init];
  mouth = [[Mouth allocFromZone:[self zone]] init];
  [eyePair setPupilSize:0.5];
  [mouth setSmileFactor:0.5];
  return self;
}


- didLoadNib
{
  [eyePair setPupilSize:[pupilSizeSlider floatValue]];
  [mouth setSmileFactor:[smileFactorSlider floatValue]];
  return self;
}


- drawSelf:(const NXRect *)rects :(int)numRects
{
  NXPoint             center;
  NXCoord             radius;
  NXRect              subRect;
  
  NXSetColor(backgroundColor);
  NXRectFill(&bounds);
  NXSetColor(foregroundColor);
  
  center.x = bounds.origin.x + bounds.size.width / 2.0;
  center.y = bounds.origin.y + bounds.size.height / 2.0;
  radius = MIN(bounds.size.width, bounds.size.height) / 2.0;
  PSsetlinewidth(radius / LINEWIDTHFACTOR);
  PSsetlinecap(1);
  
  subRect.origin.x = center.x - radius;
  subRect.origin.y = center.y;
  subRect.size.width = radius * 2.0;
  subRect.size.height = radius;
  [eyePair drawSelf:&subRect];
  subRect.origin.y = center.y - radius;
  [mouth drawSelf:&subRect];
  
  PSarc(center.x, center.y, radius, 0.0, 360.0);
  PSstroke();
  
  return self;
}


- takePupilSizeFrom:sender
{
  [eyePair setPupilSize:[pupilSizeSlider floatValue]];
  [self update];
  return self;
}


- takeSmileFactorFrom:sender
{
  [mouth setSmileFactor:[smileFactorSlider floatValue]];
  [self update];
  return self;
}


@end
