/**------------------------------------*------------------------------------**
    RELEASED TO THE PUBLIC DOMAIN BY CODEWORKS DEVELOPMENT, DECEMBER 1993.
    You may freely copy, distribute and reuse the code in this example. Codeworks disclaims any warranty of any kind, expressed or implied, as to its fitness for any particular use.
    Author:  Andrew Vyrros, av@codeworks.com
 **------------------------------------*------------------------------------**/
/**------------------------------------*------------------------------------**
    File:    Manager.h
    Project: DynaDoodle
    Info:    Interface of the Manager class. Manager acts as the central object for the DynaDoodle application, and is NXApp's delegate.  It coordinates the loading of doodle modules and the selection of the current doodle. It also forwards some graphical control methods to the current doodle.
    In its implementation, Manager is tied explicitly to DoodleModule and Doodle. It would have been possible to create a version with far less dependence on these classes. Such a version would have made it easier to use Manager in other projects, but I felt that this would have obscured the concepts and constructs of dynamic loading. I hope that once you understand the basic structure, you can easily adapt it to your own uses.
 **------------------------------------*------------------------------------**/

#import <objc/Object.h>

@class List, Doodle;


@interface Manager:Object
{
  /* nib outlets */
  id                  doodleBox;            // Box to place doodle in
  id                  currentDoodleButton;  // cover of currentDoodlePopUpList
  id                  descriptionTextField; // TextField for doodle description
  id                  foregroundColorWell;  // NXColorWells for doodle colors
  id                  backgroundColorWell;
  id                  customControlBox;     // Box to place doodle controls in
  id                  infoPanel;            // Info Panel
  
  List               *bundleList;           // List of DoodleBundles
  Doodle             *currentDoodle;        // currently selected doodle
}

- appDidInit:sender;
- free;
- createBundlesAndLoadModules:(BOOL)doLoad;
- createBundlesForDirectory:(const char *)dirPath loadModules:(BOOL)doLoad;
- fillCurrentDoodlePopUpList;
- setCurrentDoodleToDefault;
- setCurrentDoodle:(Doodle *)newDoodle;
- takeCurrentDoodleFrom:sender;
- takeForegroundColorFrom:sender;
- takeBackgroundColorFrom:sender;
- showInfoPanel:sender;

@end
