/**------------------------------------*------------------------------------**
    RELEASED TO THE PUBLIC DOMAIN BY CODEWORKS DEVELOPMENT, DECEMBER 1993.
    You may freely copy, distribute and reuse the code in this example. Codeworks disclaims any warranty of any kind, expressed or implied, as to its fitness for any particular use.
    Author:  Andrew Vyrros, av@codeworks.com
 **------------------------------------*------------------------------------**/
/**------------------------------------*------------------------------------**
    File:    Sine.m
    Project: DynaDoodle/Sine.bproj
    Info:    Implementation of the Sine class.
 **------------------------------------*------------------------------------**/

#import "Sine.h"
#import <appkit/appkit.h>


#define NAME NXLocalizedStringFromTableInBundle("Doodle.strings", [NXBundle bundleForClass:[Sine class]], "Name", "Sine Wave", "Name of Sine Doodle")
#define DESCRIPTION NXLocalizedStringFromTableInBundle("Doodle.strings", [NXBundle bundleForClass:[Sine class]], "Description", "A sine wave. You can manipulate the frequency (number of cycles) and the amplitude (height) of the wave.", "Description of Sine Doodle")
#define XSTEP        1.0


@implementation Sine


- initFrame:(const NXRect *)f
{
  [super initFrame:f];
  frequency = 1.0;
  amplitude = 1.0;
  lineWidth = 1.0;
  return self;
}


- didLoadNib
{
  frequency = [frequencySlider floatValue];
  amplitude = [amplitudeSlider floatValue];
  lineWidth = [lineWidthSlider floatValue];
  return self;
}


- drawSelf:(const NXRect *)rects :(int)numRects
{
  float               t, tStep;
  int                 numSteps;
  NXPoint             p;
  NXCoord             yStart, yFactor;
  
  NXSetColor(backgroundColor);
  NXRectFill(&bounds);
  NXSetColor(foregroundColor);
  
  
  PSsetlinewidth(lineWidth);

  numSteps = bounds.size.width / XSTEP;
  tStep = frequency * 2 * M_PI / numSteps;
  yStart = bounds.size.height / 2.0;
  yFactor = amplitude * bounds.size.height / 2.0;

  t = 0.0;
  p.x = bounds.origin.x;
  p.y = yStart;
  PSmoveto(p.x, p.y);
  while (numSteps--)
  {
    t += tStep;    
    p.x += XSTEP;
    p.y = yStart + yFactor * sin(t);
    PSlineto(p.x, p.y);
  }
  PSstroke();
  return self;
}


- takeFreqencyFrom:sender
{
  frequency = [[sender selectedCell] floatValue];
  [self update];
  return self;
}


- takeAmplitudeFrom:sender
{
  amplitude = [[sender selectedCell] floatValue];
  [self update];
  return self;
}


- takeLineWidthFrom:sender
{
  lineWidth = [[sender selectedCell] floatValue];
  [self update];
  return self;
}


@end
