/**------------------------------------*------------------------------------**
    RELEASED TO THE PUBLIC DOMAIN BY CODEWORKS DEVELOPMENT, DECEMBER 1993.
    You may freely copy, distribute and reuse the code in this example. Codeworks disclaims any warranty of any kind, expressed or implied, as to its fitness for any particular use.
    Author:  Andrew Vyrros, av@codeworks.com
 **------------------------------------*------------------------------------**/
/**------------------------------------*------------------------------------**
    File:    Spiral.m
    Project: DynaDoodle/Spiral.bproj
    Info:    Implementation of the Spiral class.
 **------------------------------------*------------------------------------**/

#import "Spiral.h"
#import <appkit/appkit.h>


#define NAME NXLocalizedStringFromTableInBundle("Doodle.strings", [NXBundle bundleForClass:[Spiral class]], "Name", "Spiral", "Name of Spiral Doodle")
#define DESCRIPTION NXLocalizedStringFromTableInBundle("Doodle.strings", [NXBundle bundleForClass:[Spiral class]], "Description", "A psuedo-spiral curve. You have control over the number of loops and the line width.", "Description of Spiral Doodle")


@implementation Spiral


- initFrame:(const NXRect *)f
{
  [super initFrame:f];
  numLoops = 10.0;
  lineWidth = 1.0;
  return self;
}


- didLoadNib
{
  numLoops = [numLoopsSlider intValue];
  lineWidth = [lineWidthSlider floatValue];
  return self;
}


- drawSelf:(const NXRect *)rects :(int)numRects
{
  NXPoint             step;
  NXCoord             xCenter,
                      l,
                      r,
                      b,
                      t;
  int                 loop;
  
  NXSetColor(backgroundColor);
  NXRectFill(&bounds);
  NXSetColor(foregroundColor);

  PSsetlinewidth(lineWidth);
  
  step.x = bounds.size.width / numLoops / 2.0;
  step.y = bounds.size.height / numLoops / 2.0;
  xCenter = bounds.origin.x + bounds.size.width / 2.0;
  l = bounds.origin.x - step.x * 0.25;
  r = bounds.origin.x + bounds.size.width + step.x * 0.75;
  b = bounds.origin.y - step.y * 0.5;
  t = bounds.origin.y + bounds.size.height;
  
  loop = numLoops;
  while (loop--)
  {
    r -= step.x;
    b += step.y;
    PSmoveto(xCenter, t);
    PScurveto(r, t, r, b, xCenter, b);
    PSstroke();
    
    l += step.x;
    t -= step.y;
    PSmoveto(xCenter, b);
    PScurveto(l, b, l, t, xCenter, t);  
    PSstroke();
  }
  
  return self;
}


- takeNumLoopsFrom:sender
{
  numLoops = [[sender selectedCell] intValue];
  [self update];
  return self;
}


- takeLineWidthFrom:sender
{
  lineWidth = [[sender selectedCell] floatValue];
  [self update];
  return self;
}


@end
