// InspectorManager.h
// By Kevin Brain
// A class for managing an inspector.
// Compositing techniques and functions from ToolInspector example
// by Sharon Biocca Zakhour, NeXT Developer Support Team
// You may freely copy, distribute and reuse the code in this example.  
// *Everybody involved* disclaims any warranty of any kind, expressed 
// or implied, as to its fitness for any particular use.

#import <objc/Object.h>

/* Inspector Type */
#define NOSELECTIONS 0
#define MULTIPLESELECTIONS 1
#define NOTAPPLICABLE 2
#define FIRSTADDEDINSPECTOR 3
#define LOWERLEFTX 10
#define LOWERLEFTY 40

@interface InspectorManager:Object
{
    id	inspectorPanel;
    id	layouts;
    id	inspectorStrings;
	id	noInspectorBox;
	id	multiInspectorBox;
	id	unapplicableInspectorBox;
	id	delegate;
	id	revertOKOut;
	id	popupOut;		// attaches to button that covers popup list
	id	inspectorList;
	id	groupList;
    id	visibleInspectors;	/* Storage of visible inspector numbers */
	NXRect	lastRect;
	BOOL useKeyEquivalents;
	struct inspectorListEntry {	// list of inspector info
		id view;				// the box composing the inspector
		char *title;			// pointer to inspector title
		BOOL showing;			// YES if showing in the panel
		NXRect offscreenRect;	// rect in layout view for compositing
		};	
}

/* instance methods */
- init;
- (unsigned int)addInspector:(id)theView title:(const char *)theTitle;
- (unsigned int)addInspector:(id)theView title:(const char *)theTitle 
	atLocation:(NXCoord)xLoc :(NXCoord)yLoc;
- (unsigned int)addGroup:(const char *)theTitle;
- setUseKeyEquivalents:(BOOL)use;
- switchToInspector:(unsigned int)newInspector;
- showInspector:(unsigned int)newInspector;
- hideInspector:(unsigned int)inspectorNum;
- (BOOL)showing:(unsigned int)inspectorNum;
- (int)group;
- panel;
- popUpListButton;
- revertOKMatrix;
- setDelegate:(id)anObject;
- delegate;
- selectGroup:sender;
- revertPressed:sender;
- okPressed:sender;
- showRevertOK;
- hideRevertOK;
- showGroupPopUp;
- hideGroupPopUp;
@end

@interface Object(InspectorManagerDelegate)
- groupChanged:sender to:(int)newGroup;
- inspectRevert:sender;
- inspectOK:sender;
@end

