/*
 *	Written by Joe Freeman jfreeman@next.com July 1991.
 *	Use this object at your own risk.  
 *	No support or implication of fitness for use is stated or implied.
 *
 *	InspectorPanel:
 *		This object implements a standard inspector panel, similar to
 *		the inspector panel in InterfaceBuilder.  Inspectors have three 
 *		important parts that define the "inspector" part of its 
 *		functionallity, 
 *			A control to determine current inspector
 *			View where variable part of inspector is be displayed.
 *			View that will be scrolled around in variable part.
 *		This object allows the programmer to make connections to all 
 *		elements of the inspector inside a single nib file. It assumes
 *		that the there is a view that acts as the template for the size
 *		of the changable area, an offscreen window that holds the 
 *		images of the of the inspectors, and some kind of matrix of 
 *		buttons that will determine the current inspector, usually
 *		a popup list.
 *
 *		This Panel assumes that all of the inspectors are tiled in
 *		a single window.  That window (contentView) is scrolled around
 *		behind a ClipView that is the size of a single inspector.
 *
 *		All other functionality of the InspectorPanel is identical to
 *		that of standard Panels.  Any number of non changable views
 *		can coexist with the inspection area.
 *
 *		inspectorView
 *				 is the view where the information will be 
 *		displayed in the inspectorPanel.  It is a ClipView that
 *		gives the InspectorPanel size and location information.
 *
 *		offscreenWindow 
 *				is the window who's content view will be 
 *		moved around inside the inspectorView.  The order of the views
 *		(in reference to the order of the popup list is 
 *			.
 *			.
 *			.
 *			m+1 m+2 m+3 ...
 *			n+1 n+2 n+3 ...
 *			0   1   2
 *		setup the offscreenWindow as:
 *			NOT visible on launch
 *			NO resizebar
 *			NOT deferred
 *
 *
 *		The message 
 *			-updateInspector:
 *		causes the sender to be checked to figure out how the offscreen
 *		window should be displayed in the clipview and notifies the
 *		the delegate of the change.
 *
 */

#import <appkit/Panel.h>

@interface InspectorPanel:Panel
{
    id	inspectorView;
    id	offscreenWindow;	/* changed in code to point to content view */
}

/*
 * This message should be sent by all elements of the pop up list. 
 * Tags are not used, the location of the cell in the matrix is used.
 */
- updateInspector:sender;

@end

@interface InspectorPanelDelegate:Object
{
}

/*
 * This method is sent to the delegate whenever the current inspector changes
 * It sends the index of the current inspector (similar to tag when tag = pos)
 */
- inspectorDidChangeTo:(int)inspectNum;
@end