// MODocType.h
//
// by Mike Ferris
// Part of MOKit
// Copyright 1993, all rights reserved.

// ABOUT MOKit
// 
// MOKit is a collection of useful and general objects.  Permission is 
// granted by the author to use MOKit in your own programs in any way 
// you see fit.  All other rights to the kit are reserved by the author 
// including the right to sell these objects as part of a LIBRARY or as 
// SOURCE CODE.  In plain English, I wish to retain rights to these 
// objects as objects, but allow the use of the objects as pieces in a 
// fully functional program.  NO WARRANTY is expressed or implied.  The author 
// will under no circumstances be held responsible for ANY consequences to 
// you from the use of these objects.  Since you don't have to pay for 
// them, and full source is provided, I think this is perfectly fair.

// ABOUT MODocType
//
// MODocType is small data-class used by MODocController.  It stores a 
// document tag, a name, a file extension, and two selectors pointing
// to the revert and save methods to be called in the controller for the
// given type.

#import "MOController.h"

// Forward declarations of class names we'll use for static typing.
@class MOString;

@interface MODocType:Object
{
	Class controllerClass;
	int typeTag;
	MOString *typeName;
	MOString *typeExtension;
	SEL openSelector;
	SEL saveSelector;
}

+ initialize;

- init;
- initForControllerClass:(Class)controller tag:(int)tag 
			name:(const char *)name extension:(const char *)ext 
			openSelector:(SEL)openSel saveSelector:(SEL)saveSel;
- free;

- (Class)controllerClass;
- (int)typeTag;
- (const char *)typeName;
- (const char *)typeExtension;
- (BOOL)canOpenType;
- (SEL)openSelector;
- (BOOL)canSaveType;
- (SEL)saveSelector;

- read:(NXTypedStream *)strm;
- write:(NXTypedStream *)strm;

@end

