// MOImageDocController.m
//
// by Mike Ferris
// Part of MOKit
// Copyright 1993, all rights reserved.

#import "MOImageDocController.h"
#import "MOKit/MOPathString.h"

#define CLASS_VERSION			0

#define CLASS_NIB				"MOImageDocController.nib"

@implementation MOImageDocController

+ initialize
// Set the version.
{
	if (self == [MOImageDocController class])  {
		[self setVersion:CLASS_VERSION];
		
		[self setClassNib:CLASS_NIB];
		[self setControllerName:"Image"];
		[self addDocType:1 name:"TIFF Image" extension:"tiff" 
			openSelector:@selector(doRevertImage:) 
			saveSelector:@selector(doSaveTiff:)];
		[self addDocType:2 name:"EPS Image" extension:"eps" 
			openSelector:@selector(doRevertImage:) 
			saveSelector:NULL];
	}
	return self;
}

- nibDidLoad
{
	[super nibDidLoad];
	
	return self;
}

- doSaveTiff:(MODocType *)type
{
	NXStream *strm = NULL;
	id ret = self;

	if (![self image])  {
		return nil;
	}
	
	strm = NXOpenMemory(NULL, 0, NX_WRITEONLY);
	if (strm)  {
		[[self image] writeTIFF:strm allRepresentations:YES 
					usingCompression:NX_TIFF_COMPRESSION_LZW 
					andFactor:0.0];
		if (NXSaveToFile(strm, [self file]) == -1)  {
			ret = nil;
		}
		
		NXCloseMemory(strm, NX_FREEBUFFER);
	}  else  {
		ret = nil;
	}
	
	return ret;
}

- doRevertImage:(MODocType *)type
{
	id ret = self;
	
	id newImage = [[NXImage allocFromZone:[self zone]] 
				initFromFile:[self file]];
	if (newImage)  {
		[self setImage:newImage];
	}  else  {
		ret = nil;
	}
	
	return ret;
}

- (BOOL)doClear
{
	[self setImage:nil];
	
	return YES;
}

- (BOOL)doPrint
{
	[imageView printPSCode:self];
	return YES;
}

- imageView
{
	return imageView;
}

- setImage:(NXImage *)image
{
	id oldImage = [self image];
	[imageView setImage:image];
	if (oldImage)  [oldImage free];
	return self;
}

- image
{
	return [imageView image];
}

@end

