// MOTextDocController.m
//
// by Mike Ferris
// Part of MOKit
// Copyright 1993, all rights reserved.

#import "MOTextDocController.h"
#import "MOKit/MOPathString.h"

#define CLASS_VERSION			0

#define CLASS_NIB				"MOTextDocController.nib"

@implementation MOTextDocController

+ initialize
// Set the version.
{
	if (self == [MOTextDocController class])  {
		[self setVersion:CLASS_VERSION];
		
		[self setClassNib:CLASS_NIB];
		[self setControllerName:"Rich Text"];
		[self addDocType:1 name:"Rich Text" extension:"rtf" 
			openSelector:@selector(doRevertRtf:) 
			saveSelector:@selector(doSaveRtf:)];
		[self addDocType:2 name:"Text Doc" extension:"txt" 
			openSelector:@selector(doRevertTxt:) 
			saveSelector:@selector(doSaveTxt:)];
		[self addDocType:3 name:"Raw Man Page" extension:"1" 
			openSelector:@selector(doRevertTxt:) 
			saveSelector:NULL];
		[self addDocType:4 name:"Ascii Text" extension:NULL
			openSelector:@selector(doRevertTxt:) 
			saveSelector:@selector(doSaveTxt:)];
	}
	return self;
}

- nibDidLoad
{
	[super nibDidLoad];
	
	[textObj setDelegate:self];
	
	return self;
}

- doSaveRtf:(MODocType *)type
{
	NXStream *strm = NULL;
	id ret = self;
	
	strm = NXOpenMemory(NULL, 0, NX_WRITEONLY);
	if (strm)  {
		[textObj setMonoFont:NO];
		[textObj writeRichText:strm];
		
		if (NXSaveToFile(strm, [self file]) == -1)  {
			ret = nil;
		}
		
		NXCloseMemory(strm, NX_FREEBUFFER);
	}  else  {
		ret = nil;
	}

	return ret;
}

- doSaveTxt:(MODocType *)type
{
	NXStream *strm = NULL;
	id ret = self;
	
	strm = NXOpenMemory(NULL, 0, NX_WRITEONLY);
	if (strm)  {
		[textObj setMonoFont:YES];
		[textObj setFont:[Font userFixedPitchFontOfSize:12.0 
					matrix:NX_FLIPPEDMATRIX]];
		[textObj writeText:strm];
		
		if (NXSaveToFile(strm, [self file]) == -1)  {
			ret = nil;
		}
		
		NXCloseMemory(strm, NX_FREEBUFFER);
	}  else  {
		ret = nil;
	}

	return ret;
}

- doRevertRtf:(MODocType *)type
{
	NXStream *strm = NULL;
	id ret = self;
	
	strm = NXMapFile([self file], NX_READONLY);
	if (strm)  {
		[textObj setMonoFont:NO];
		[textObj readRichText:strm];
		NXCloseMemory(strm, NX_FREEBUFFER);
	}  else  {
		ret = nil;
	}
	
	return ret;
}

- doRevertTxt:(MODocType *)type
{
	NXStream *strm = NULL;
	id ret = self;
	
	strm = NXMapFile([self file], NX_READONLY);
	if (strm)  {
		[textObj setMonoFont:YES];
		[textObj setFont:[Font userFixedPitchFontOfSize:12.0 
					matrix:NX_FLIPPEDMATRIX]];
		[textObj readText:strm];
		NXCloseMemory(strm, NX_FREEBUFFER);
	}  else  {
		ret = nil;
	}
	
	return ret;
}

- (BOOL)doClear
{
	if (![super doClear])  return NO;
	
	[textObj setText:""];
	[textObj setMonoFont:NO];
	return YES;
}

- (BOOL)doPrint
{
	if (![super doClear])  return NO;

	[textObj printPSCode:self];
	return YES;
}

- textObj
{
	return textObj;
}

- textDidGetKeys:sender isEmpty:(BOOL)flag
{
	[self setDirty:YES];
	return self;
}


@end

