// MOImageDocController.m
//
// by Mike Ferris
// Part of MOKit
// Copyright 1993, all rights reserved.

#import "MOImageDocController.h"
#import "MOKit/MOPathString.h"

#define CLASS_VERSION			0

#define CLASS_NIB				"MOImageDocController.nib"

@implementation MOImageDocController

+ initialize
// Set the version.
{
	if (self == [MOImageDocController class])  {
		[self setVersion:CLASS_VERSION];
		
		[self setClassNib:CLASS_NIB];
		[self setDocTypeName:"Image"];
		[self addOpenType:"tiff"];
		[self addOpenType:"eps"];
		[self addSaveType:"tiff"];
//		[self addSaveType:"eps"];
	}
	return self;
}

- nibDidLoad
{
	[super nibDidLoad];
	
	return self;
}

- (BOOL)doSave
{
	NXStream *strm = NULL;
	id extension = [docName fileExtension];
	BOOL ret = YES;

	if (![self image])  {
		return NO;
	}
	
	if ([super doSave])  {
		strm = NXOpenMemory(NULL, 0, NX_WRITEONLY);
		if (strm)  {
			[[self image] writeTIFF:strm allRepresentations:YES 
						usingCompression:NX_TIFF_COMPRESSION_LZW 
						andFactor:0.0];
			if (NXSaveToFile(strm, [self file]) == -1)  {
				ret = NO;
			}
			
			NXCloseMemory(strm, NX_FREEBUFFER);
		}  else  {
			ret = NO;
		}
	}  else  {
		ret = NO;
	}
	
	[extension free];

	return ret;
}

- (BOOL)doRevert
{
	id extension = [docName fileExtension];
	BOOL ret = YES;
	
	if ([super doRevert])  {
		id newImage = [[NXImage allocFromZone:[self zone]] 
					initFromFile:[self file]];
		if (newImage)  {
			[self setImage:newImage];
		}  else  {
			ret = NO;
		}
	}  else  {
		ret = NO;
	}
	
	[extension free];
	
	return ret;
}

- (BOOL)doClear
{
	if (![super doClear])  return NO;
	
	[self setImage:nil];
	
	return YES;
}

- (BOOL)doPrint
{
	if (![super doClear])  return NO;

	[imageView printPSCode:self];
	return YES;
}

- imageView
{
	return imageView;
}

- setImage:(NXImage *)image
{
	id oldImage = [self image];
	[imageView setImage:image];
	if (oldImage)  [oldImage free];
	return self;
}

- image
{
	return [imageView image];
}

@end

