// MOTextDocController.m
//
// by Mike Ferris
// Part of MOKit
// Copyright 1993, all rights reserved.

#import "MOTextDocController.h"
#import "MOKit/MOPathString.h"

#define CLASS_VERSION			0

#define CLASS_NIB				"MOTextDocController.nib"

@implementation MOTextDocController

+ initialize
// Set the version.
{
	if (self == [MOTextDocController class])  {
		[self setVersion:CLASS_VERSION];
		
		[self setClassNib:CLASS_NIB];
		[self setDocTypeName:"Rich Text"];
		[self addOpenType:"rtf"];
		[self addOpenType:"1"];
		[self addOpenType:"txt"];
		[self addSaveType:"rtf"];
		[self addSaveType:"txt"];
	}
	return self;
}

- nibDidLoad
{
	[super nibDidLoad];
	
	[textObj setDelegate:self];
	
	return self;
}

- (BOOL)doSave
{
	NXStream *strm = NULL;
	id extension = [docName fileExtension];
	BOOL ret = YES;
	
	if ([super doSave])  {
		strm = NXOpenMemory(NULL, 0, NX_WRITEONLY);
		if (strm)  {
			if ([extension compareStr:"rtf"] == 0)  {
				[textObj writeRichText:strm];
			}  else  {
				[textObj writeText:strm];
			}
			
			if (NXSaveToFile(strm, [self file]) == -1)  {
				ret = NO;
			}
			
			NXCloseMemory(strm, NX_FREEBUFFER);
		}  else  {
			ret = NO;
		}
	}  else  {
		ret = NO;
	}
	
	[extension free];

	return ret;
}

- (BOOL)doRevert
{
	NXStream *strm = NULL;
	id extension = [docName fileExtension];
	BOOL ret = YES;
	
	if ([super doRevert])  {
		strm = NXMapFile([self file], NX_READONLY);
		if (strm)  {
			if ([extension compareStr:"rtf"] == 0)  {
				[textObj readRichText:strm];
			}  else  {
				[textObj readText:strm];
			}
			NXCloseMemory(strm, NX_FREEBUFFER);
		}  else  {
			ret = NO;
		}
	}  else  {
		ret = NO;
	}
	
	[extension free];
	
	return ret;
}

- (BOOL)doClear
{
	if (![super doClear])  return NO;
	
	[textObj setText:""];
	[textObj setMonoFont:NO];
	return YES;
}

- (BOOL)doPrint
{
	if (![super doClear])  return NO;

	[textObj printPSCode:self];
	return YES;
}

- textObj
{
	return textObj;
}

- textDidGetKeys:sender isEmpty:(BOOL)flag
{
	[self setDirty:YES];
	return self;
}


@end

