// Copyright (c) 1995 Montgomery Zukowski
// Use is governed by the MiscKit license

#import <remote/NXConnection.h>
#import <remote/NXProxy.h>
#import <apps/InterfaceBuilder.h>

#import "../IBMole.h"

static const char *fullPath(char *path, const char *currentDir,
	const char *nib);
static void modifyNib(id workspace, const char *fullPath, id iBApp);
static void modifyDocument(id <IBDocuments> activeDocument);
static void modifyDocumentObjects(id <IBDocuments> activeDocument);

@protocol Proxy <IBSecretAgent>

// Add proxy protocol.

@end

@protocol IBApp <IB>

- saveDocument:sender;
- closeDocument:sender;

@end

void main(int argc, char *argv[])
{
	unsigned nextArg = 1;
	unsigned indx = nextArg;
	id <NXWorkspaceRequestProtocol> workspace = [Application workspace];
	id server;
	char currentDir[MAXPATHLEN];

	getwd(currentDir);

	// IB can be slow to launch with lots of palettes, and complex nibs can be
	// slow to open which can result in a NX_receiveTimedOut exeption (11013)
	// being raised, so increase timeout to 45 seconds.
	[NXConnection setDefaultTimeout:45000];

	// Should IB crash, catch D.O. timeout exception and loop through arguments
	// until all have been processed.
	while (indx < argc)
	{
	NX_DURING
		// Launch IB if not already running and connect to IBMole (must be on
		// IB's palette, but it need not be the current palette).
		if ([workspace launchApplication:"InterfaceBuilder"] &&
			(server = [NXConnection connectToName:"IBMole"]) != nil)
		{
			[server setProtocolForProxy:@protocol(Proxy)];

			// Loop through all command-line arguments.
			for (indx = nextArg; indx < argc; indx++)
			{
				// Ignore any argument that's not a nib name.
				if (strstr(argv[indx], ".nib") != NULL)
				{
					char path[MAXPATHLEN];

					modifyNib(workspace, fullPath(path, currentDir,
						(const char *)argv[indx]), [server ibApp]);
				}
			}
		}
	NX_HANDLER
		// If a timeout occurs, assume IB has crashed and loop back to restart
		// IB with the current argument being the first one to process.
		if (NXLocalHandler.code == NX_receiveTimedOut)
		{
			nextArg = indx;
		}
		// Punt for any other exceptions.
		else
		{
			NX_RERAISE();
		}
	NX_ENDHANDLER
	}
	exit(0);
}

static const char *fullPath(char *path, const char *currentDir,
	const char *nib)
{
	// Construct proper path for various input formats.
	switch (nib[0])
	{
		case '~':
			sprintf(path, "%s%s", NXHomeDirectory(), nib + 1);
			return (const char *)path;
		case '/':
			return nib;
		default:
			sprintf(path, "%s/%s", currentDir, nib);
			return (const char *)path;
	}
}

static void modifyNib(id workspace, const char *fullPath, id iBApp)
{
	// Open nib in IB.
	if ([workspace openFile:fullPath])
	{
		id <IBDocuments> activeDocument;

		// Loading nib can take some time, so loop until active document
		// exists.
		while ((activeDocument = [iBApp activeDocument]) == nil)
		{
			sleep(1);
		}
		modifyDocument(activeDocument);
		[iBApp saveDocument:nil];
		[iBApp closeDocument:nil];
	}
}

static void modifyDocument(id <IBDocuments> activeDocument)
{
	// Add any custom document modifications here.

	modifyDocumentObjects(activeDocument);
}

static void modifyDocumentObjects(id <IBDocuments> activeDocument)
{
	unsigned indx;
	List *objs = [[List alloc] init];
	id obj;

	[activeDocument getObjects:objs];

	// Loop through all objects in nib.
	for (indx = 0; (obj = [objs objectAt:indx]) != nil; indx++)
	{
		// Add any custom object modifications here.
	}
	objs = [objs free];
}
