//
//	MiscValueCell.h -- a TextFieldCell subclass that adds arrow buttons
//	   Written by David Fedchenko. Copyright 1994 by David Fedchenko.
//	       		 Version 2.0  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <appkit/appkit.h>

@interface MiscValueCell:TextFieldCell
    {
    id      (idUp);
    id      (idDown);
    
    id      stringList;
    
    double  actualValue;
    double  limitMinValue;
    double  limitMaxValue;
    double  limitMinBound;
    double  limitMaxBound;
    double  sizeStep;
    double  sizeAltStep;
    BOOL    fExpandLow;
    BOOL    fExpandHigh;
    
    BOOL    fLoop;
    BOOL    fValidRect;
    int     fAltKeyDown;
    NXRect  rectFrame;
    }

- initTextCell:(const char *)sz;
- copyFromZone:(NXZone *)zone;

- drawInside:(const NXRect *)aRect inView:controlView;
- drawSelf:(const NXRect *)cellFrame inView:controlView;
- resetCursorRect:(const NXRect *)cellFrame inView:aView;

- calcCellSize:(NXSize *)theSize inRect:(const NXRect *)aRect;
- edit:(const NXRect *)aRect inView:controlView editor:textObj
    delegate:anObject event:(NXEvent *)theEvent;
- getDrawRect:(NXRect *)theRect;
- getTitleRect:(NXRect *)theRect;
- highlight:(const NXRect *)cellFrame inView:aView lit:(BOOL)flag;
- select:(const NXRect *)aRect inView:aView editor:aTextObject
    delegate:anObject start:(int)selStart length:(int)selLength;
- setBezeled:(BOOL)flag;
- setBordered:(BOOL)flag;
- setEditable:(BOOL)flag;

- setDoubleValue:(double)aDouble;
- setFloatValue:(float)aFloat;
- setIntValue:(int)anInt;
- setStringValue:(const char *)aString;
-(int) intValue;
-(float) floatValue;
-(double) doubleValue;

- increment:sender;
- decrement:sender;

- setMinValue:(double)value;
- setMaxValue:(double)value;
- setMinBoundary:(double)value;
- setMaxBoundary:(double)value;
-(double) minValue;
-(double) maxValue;
-(double) minBoundary;
-(double) maxBoundary;
- setStepSize:(double)size;
- setAltStepSize:(double)size;
-(double) stepSize;
-(double) altStepSize;
- setExpandMin:(BOOL)flag;
- setExpandMax:(BOOL)flag;
-(BOOL) expandMin;
-(BOOL) expandMax;

- setStringList:anObject;
- stringList;

- write:(NXTypedStream *)stream;
- read:(NXTypedStream *)stream;
- awake;
- awakeFromNib;

@end

@interface Object(MiscValueDelegate)
-(const char *) stringAt:(int)item;
@end
