//
//	MiscAbstraction.m -- A root object for classes without instances.
//		Written by Michael T. H. Scott Copyright 1995 by Michael T. H. Scott.
//				Version 1.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/MiscAbstraction.h>
#import <objc/Protocol.h>

@implementation MiscAbstraction

+ initialize
{
	return self;
}

+ (const char *)name
{
	return object_getClassName(self);
}

+ class
{
	return self;
}

+ superclass
{
	return ((Class)self)->super_class;
}

+ (BOOL)conformsTo: (Protocol *)aProtocol
{
	struct objc_protocol_list *protocolList = ((Class)self)->protocols;
	
	while ( protocolList )
	{
		int i;
		
		for ( i = 0; i < protocolList->count; i++ )
		{
			if ( protocolList->list[i] == aProtocol ) return YES;
		}
		
		protocolList = protocolList->next;
	}
		
	return NO;
}

+ poseAs: aClassObject
{
	return class_poseAs(self, aClassObject);
}

@end