//
//	MiscDocType.m -- encapsulating class for document info
//		Written by Mike Ferris Copyright (c) 1993, 1995 by Mike Ferris.
//		Modified for use in the MiscKit by Don Yacktman from MODocType.
//				Version 1.0.  All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/MiscDocType.h>
#import <misckit/MiscString.h>
#import <objc/objc-runtime.h>

#define CLASS_VERSION						0
#define CLASS_NAME							"MiscDocType"


@implementation MiscDocType

+ initialize
// Set the version.
{
	if (self == objc_lookUpClass(CLASS_NAME))  {
		[self setVersion:CLASS_VERSION];
	}
	return self;
}

- init
// Just call the DI.
{
	return [self initForControllerClass:nil tag:-1 name:NULL extension:NULL 
				openSelector:NULL saveSelector:NULL];
}

- initForControllerClass:(Class)controller tag:(int)tag 
			name:(const char *)name extension:(const char *)ext 
			openSelector:(SEL)openSel saveSelector:(SEL)saveSel
// Designated initializer.  This is the only way to set up the ivars 
// of a MiscDocType.  There are no individual methods for setting up the ivars.
{
	[super init];
	controllerClass = controller;
	typeTag = tag;
	typeName = [[MiscString allocFromZone:[self zone]] 
				initString:name];
	typeExtension = [[MiscString allocFromZone:[self zone]] 
				initString:ext];
	openSelector = openSel;
	saveSelector = saveSel;
	return self;
}

- free
// Free our strings.
{
	[typeName free];
	[typeExtension free];
	return [super free];
}

- (Class)controllerClass
// Return the class which owns this type.
{
	return controllerClass;
}

- (int)typeTag
// Returns the type's tag.
{
	return typeTag;
}

- (const char *)typeName
// Returns the type's name.
{
	return [typeName stringValue];
}

- (const char *)typeExtension
// Returns the type's extension.
{
	return [typeExtension stringValue];
}

- (BOOL)canOpenType
// Returns whether the type can be read.
{
	return (openSelector!=NULL);
}

- (SEL)openSelector
// Returns the type's openSelector.
{
	return openSelector;
}

- (BOOL)canSaveType
// Returns whether the type can be saved.
{
	return (saveSelector!=NULL);
}

- (SEL)saveSelector
// Returns the type's saveSelector.
{
	return saveSelector;
}


- read:(NXTypedStream *)strm
// This probably is useful.  Especially since these objects are 
// generally stored in class variables.
{
	int classVersion;

	[super read:strm];
	
	classVersion = NXTypedStreamClassVersion(strm, CLASS_NAME);
	
	switch (classVersion)  {
		case 0:		// First version.
			NXReadType(strm, "i", &typeTag);
			typeName = NXReadObject(strm);
			typeExtension = NXReadObject(strm);
			NXReadType(strm, ":", &openSelector);
			NXReadType(strm, ":", &saveSelector);
			break;
		default:
			NXLogError("[%s read:] class version %d cannot read "
						"instances archived with version %d", 
						CLASS_NAME, CLASS_VERSION, classVersion);
			typeTag = -1;
			typeName = [[MiscString allocFromZone:[self zone]] init];
			typeExtension = [[MiscString allocFromZone:[self zone]] init];
			openSelector = NULL;
			saveSelector = NULL;
			break;
	}
	return self;
}

- write:(NXTypedStream *)strm
// This probably is useful.  Especially since these objects are 
// generally stored in class variables.
{
	[super write:strm];
	NXWriteType(strm, "i", &typeTag);
	NXWriteObject(strm, typeName);
	NXWriteObject(strm, typeExtension);
	NXWriteType(strm, ":", &openSelector);
	NXWriteType(strm, ":", &saveSelector);
	return self;
}

@end
