//
//	MiscTaggedImage.m -- NXImage subclass that has an integer tag
//		Written by Don Yacktman Copyright (c) 1994 by Don Yacktman.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	


#import <misckit/MiscTaggedImage.h>
#import <appkit/appkit.h>
#import <objc/objc-runtime.h>

#define CLASS_VERSION		1
#define CLASS_NAME			"MiscTaggedImage"

@interface MiscTaggedImage(private)

+ _tagHash;

@end


@implementation MiscTaggedImage

+ initialize
// Set the version.
{
	if (self == objc_lookUpClass(CLASS_NAME))  {
		[self setVersion:CLASS_VERSION];
	}
	return self;
}

+ _tagHash
{
	static id tagHash = nil;
	if (!tagHash)
		tagHash = [[HashTable allocFromZone:[self zone]] 
				initKeyDesc:"i" valueDesc:"@"];
	return tagHash;
}

+ findImageWithTag:(int)aTag
{
	return (id)[[self _tagHash] valueForKey:&aTag];
}

- setTag:(int)newTag
{
	int oldTag = tag;
	id _tagHash = [[self class] _tagHash];

	tag = newTag;
	if (self == [[self class] findImageWithTag:oldTag]) {
		[_tagHash removeKey:&oldTag];
	}
	[_tagHash insertKey:&tag value:self];
	return self;
}

- (int)tag
{
	return tag;
}

- read:(NXTypedStream *)stream
{
	int version;

	[super read:stream];
	version = NXTypedStreamClassVersion(stream, CLASS_NAME);
	if ( version == CLASS_VERSION ) {
		NXReadTypes(stream,"i",&tag);
	} else {
		NXLogError("[%s %s] - unknown version of %s in typed stream",
				[[self class] name], sel_getName(_cmd), [[self class] name]);
	}
	return self;
}

- write:(NXTypedStream *)stream
{
	[super write:stream];	
	NXWriteTypes(stream,"i",&tag);
	return self;
}

@end
