// MiscDocManager.h
//
// by Mike Ferris
// Part of MOKit
// Copyright 1993, all rights reserved.

// ABOUT MOKit
// 
// MOKit is a collection of useful and general objects.  Permission is 
// granted by the author to use MOKit in your own programs in any way 
// you see fit.  All other rights to the kit are reserved by the author 
// including the right to sell these objects as part of a LIBRARY or as 
// SOURCE CODE.  In plain English, I wish to retain rights to these 
// objects as objects, but allow the use of the objects as pieces in a 
// fully functional program.  NO WARRANTY is expressed or implied.  The author 
// will under no circumstances be held responsible for ANY consequences to 
// you from the use of these objects.  Since you don't have to pay for 
// them, and full source is provided, I think this is perfectly fair.

// ABOUT MODocManager
//
// MODocManager manages a bunch of MODocControllers.

#import <appkit/appkit.h>

// Forward declarations of class names we'll use for static typing.
@class MiscDocument;

@interface MiscDocManager:Object
{
	List *docList;
	MiscDocument *currentDoc;
	List *docClassList;
	
	int untitledCount;
	int frameCycle;
	NXPoint windowStartingLocation;
	
	// quit panel outlets
	id	quitPanel;
	id	buttonMatrix;
	
	// menu outlets
	id	docMenu;
	id	openCell;
	id	newCell;
	id	saveCell;
	id	saveAsCell;
	id	saveToCell;
	id	saveAllCell;
	id	revertCell;
	id	closeCell;
	
	Menu *newMenu;
	MenuCell *newSubmenuCell;
	Menu *openMenu;
	MenuCell *openSubmenuCell;
}

+ initialize;

- init;
- initDocumentClass:(Class)docControllerClass;
- free;
- awakeFromNib;

- new:sender;
- newFromDocumentClass:sender;
- open:sender;
- openFromDocumentClass:sender;
- openDocument:(const char *)path withClass:(Class)docClass;

- save:sender;
- saveAs:sender;
- saveTo:sender;
- saveAll:sender;
- revert:sender;
- close:sender;

- print:sender;

- getNextWindowLocation:(NXPoint *)pt;
- (int)nextUntitledNum;
- setWindowStartingLocation:(const NXPoint *)pt;

- addDocument:(MiscDocument *)aDocument;
- removeDocument:(MiscDocument *)aDocument;
- (MiscDocument *)findDocumentForWindow:aWindow;
- (List *)documentList;
- makeDocumentsPerform:(SEL)aMethod with:anArg;

- (BOOL)areDocumentsDirty;

- setCurrentDocument:(MiscDocument *)aDocument;
- currentDocument;

- addDocumentClass:(Class)docControllerClass;
- removeDocumentClass:(Class)docControllerClass;
- (List *)documentClassList;
- (int)docClassCount;

- appWillTerminate:sender;
- quitPanelStopModalAction:sender;

- windowDidBecomeKey:sender;
- windowDidResignKey:sender;

- (BOOL)menuUpdate:menuCell;

- awake;
- read:(NXTypedStream *)strm;
- write:(NXTypedStream *)strm;

@end

